# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Update the developer connect git repository link."""

import datetime

from googlecloudsdk.api_lib.developer_connect.insights_configs import insights_config
from googlecloudsdk.api_lib.util import exceptions
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.developer_connect import flags
from googlecloudsdk.command_lib.developer_connect import resource_args
from googlecloudsdk.core import log
from googlecloudsdk.core import resources


DETAILED_HELP = {
    'DESCRIPTION': """
          Update a git repository link to enable metrics collection.
          """,
    'EXAMPLES': """
          To update a git repository link to enable metrics collection, run:

            $ {command} developer-connect connections git-repository-links update <LINK> --collect-metrics
          """,
}


@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.ALPHA)
@base.DefaultUniverseOnly
class Update(base.UpdateCommand):
  """Update a git repository link to enable metrics collection."""

  @staticmethod
  def Args(parser):
    # Relevant argument.
    resource_args.AddGitRepositoryLinkResourceArg(parser, verb='update')
    flags.AddCollectMetricsArgument(parser)

  def Run(self, args):
    if not args.collect_metrics:
      log.status.Print('No updates specified.')
      return
    max_wait = datetime.timedelta(seconds=30)
    client = insights_config.InsightsConfigClient(base.ReleaseTrack.BETA)
    git_repository_link_ref = args.CONCEPTS.git_repository_link.Parse()
    project_id = git_repository_link_ref.projectsId
    location_id = git_repository_link_ref.locationsId
    repo_link_id = git_repository_link_ref.gitRepositoryLinksId
    insights_config_ref = resources.REGISTRY.Parse(
        repo_link_id + '-metrics',
        params={
            'projectsId': project_id,
            'locationsId': location_id,
            'insightsConfigsId': repo_link_id + '-metrics',
        },
        collection='developerconnect.projects.locations.insightsConfigs',
    )
    log.CreatedResource(
        'create Insights Config [{0}].'.format(insights_config_ref)
    )

    # Construct the request message
    source_config = {
        'git-repository-link': git_repository_link_ref.RelativeName()
    }
    try:
      operation = client.create(
          insight_config_ref=insights_config_ref,
          app_hub=None,
          target_projects=None,
          user_artifact_configs=None,
          source_config=source_config,
      )

    except exceptions.HttpException as e:
      log.status.Print(
          'Failed to create the insight config {}.'.format(
              insights_config_ref.RelativeName()
          )
      )
      raise e

    log.status.Print(
        'Creating the insight config {}.'.format(
            insights_config_ref.RelativeName()
        )
    )
    return client.wait_for_operation(
        operation_ref=client.get_operation_ref(operation),
        message='Waiting for operation [{}] to be completed...'.format(
            client.get_operation_ref(operation).RelativeName()
        ),
        has_result=True,
        max_wait=max_wait,
    )


Update.detailed_help = DETAILED_HELP



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Retrieves the latest recovery time for a Cloud SQL instance.
"""


import http.client

from apitools.base.py import exceptions as apitools_exceptions
from googlecloudsdk.api_lib.sql import api_util
from googlecloudsdk.api_lib.sql import exceptions
from googlecloudsdk.api_lib.sql import validate
from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions as calliope_exceptions
from googlecloudsdk.command_lib.sql import flags
from googlecloudsdk.core import properties

DESCRIPTION = ("""\

    *{command}* retrieves the latest recovery time for a Cloud SQL instance.
    This is the latest time that can be used to perform point in time recovery
    for the Cloud SQL instance.
    """)

EXAMPLES_GA = ("""\
    To retrieve the latest recovery time for an instance:

    $ {command} instance-foo

    To retrieve the latest recovery time for an instance that has been deleted:

    $ {command} instance-foo --source-instance-deletion-time '2012-11-15T16:19:00.094Z'
    """)

DETAILED_HELP = {
    'DESCRIPTION': DESCRIPTION,
    'EXAMPLES': EXAMPLES_GA,
}


@base.DefaultUniverseOnly
@base.ReleaseTracks(base.ReleaseTrack.GA)
class GetLatestRecoveryTime(base.Command):
  """Displays the latest recovery time to which a Cloud SQL instance can be restored to.
  """

  detailed_help = DETAILED_HELP

  @staticmethod
  def Args(parser):
    """Args is called by calliope to gather arguments for this command.

    Args:
      parser: An argparse parser that you can use it to add arguments that go on
        the command line after this command. Positional arguments are allowed.
    """
    parser.add_argument(
        'instance',
        completer=flags.InstanceCompleter,
        help='Cloud SQL instance ID.')
    parser.add_argument(
        '--source-instance-deletion-time',
        type=arg_parsers.Datetime.Parse,
        completer=flags.InstanceCompleter,
        required=False,
        help=(
            'The deletion time of the source instance. This is used to identify'
            ' the instance if it has been deleted.'
        ),
    )

  def Run(self, args):
    """Displays the latest recovery time to which a Cloud SQL instance can be restored to.

    Args:
      args: argparse.Namespace, The arguments that this command was invoked
        with.

    Returns:
      A timestamp representing the latest recovery time to which a Cloud SQL
      instance can be restored to.

    Raises:
      HttpException: A http error response was received while executing api
          request.
      ResourceNotFoundError: The SQL instance isn't found.
    """
    client = api_util.SqlClient(api_util.API_VERSION_DEFAULT)
    sql_client = client.sql_client
    sql_messages = client.sql_messages

    validate.ValidateInstanceName(args.instance)
    instance_ref = client.resource_parser.Parse(
        args.instance,
        params={'project': properties.VALUES.core.project.GetOrFail},
        collection='sql.instances')
    try:
      request = sql_messages.SqlProjectsInstancesGetLatestRecoveryTimeRequest(
          project=instance_ref.project, instance=instance_ref.instance)
      if args.source_instance_deletion_time:
        request.sourceInstanceDeletionTime = (
            args.source_instance_deletion_time.strftime('%Y-%m-%dT%H:%M:%S.%fZ')
        )
      return sql_client.projects_instances.GetLatestRecoveryTime(request)
    except apitools_exceptions.HttpError as error:
      if error.status_code == http.client.FORBIDDEN:
        raise exceptions.ResourceNotFoundError(
            "There's no instance found at {} or you're not authorized to "
            'access it.'.format(instance_ref.RelativeName()))
      raise calliope_exceptions.HttpException(error)


@base.DefaultUniverseOnly
@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.ALPHA)
class GetLatestRecoveryTimeBeta(base.Command):
  """Displays the latest recovery time to which a Cloud SQL instance can be restored to.
  """

  detailed_help = DETAILED_HELP

  @staticmethod
  def Args(parser):
    """Args is called by calliope to gather arguments for this command.

    Args:
      parser: An argparse parser that you can use it to add arguments that go on
        the command line after this command. Positional arguments are allowed.
    """
    parser.add_argument(
        'instance',
        completer=flags.InstanceCompleter,
        help='Cloud SQL instance ID.')
    parser.add_argument(
        '--source-instance-deletion-time',
        type=arg_parsers.Datetime.Parse,
        completer=flags.InstanceCompleter,
        required=False,
        help=(
            'The deletion time of the source instance. This is used to identify'
            ' the instance if it has been deleted.'
        ),
    )

  def Run(self, args):
    """Displays the latest recovery time to which a Cloud SQL instance can be restored to.

    Args:
      args: argparse.Namespace, The arguments that this command was invoked
        with.

    Returns:
      A timestamp representing the latest recovery time to which a Cloud SQL
      instance can be restored to.

    Raises:
      HttpException: A http error response was received while executing api
          request.
      ResourceNotFoundError: The SQL instance isn't found.
    """
    client = api_util.SqlClient(api_util.API_VERSION_DEFAULT)
    sql_client = client.sql_client
    sql_messages = client.sql_messages

    validate.ValidateInstanceName(args.instance)
    instance_ref = client.resource_parser.Parse(
        args.instance,
        params={'project': properties.VALUES.core.project.GetOrFail},
        collection='sql.instances')
    try:
      request = sql_messages.SqlProjectsInstancesGetLatestRecoveryTimeRequest(
          project=instance_ref.project, instance=instance_ref.instance)
      if args.source_instance_deletion_time:
        request.sourceInstanceDeletionTime = (
            args.source_instance_deletion_time.strftime('%Y-%m-%dT%H:%M:%S.%fZ')
        )
      return sql_client.projects_instances.GetLatestRecoveryTime(request)
    except apitools_exceptions.HttpError as error:
      if error.status_code == http.client.FORBIDDEN:
        raise exceptions.ResourceNotFoundError(
            "There's no instance found at {} or you're not authorized to "
            'access it.'.format(instance_ref.RelativeName()))
      raise calliope_exceptions.HttpException(error)



# -*- coding: utf-8 -*- #
# Copyright 2014 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""deployments update command."""


from apitools.base.py import exceptions as apitools_exceptions

from googlecloudsdk.api_lib.deployment_manager import dm_api_util
from googlecloudsdk.api_lib.deployment_manager import dm_base
from googlecloudsdk.api_lib.deployment_manager import dm_labels
from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.command_lib.deployment_manager import alpha_flags
from googlecloudsdk.command_lib.deployment_manager import dm_util
from googlecloudsdk.command_lib.deployment_manager import dm_write
from googlecloudsdk.command_lib.deployment_manager import flags
from googlecloudsdk.command_lib.deployment_manager import importer
from googlecloudsdk.command_lib.util.apis import arg_utils
from googlecloudsdk.command_lib.util.args import labels_util
from googlecloudsdk.core import log
from googlecloudsdk.core import properties

# Number of seconds (approximately) to wait for update operation to complete.
OPERATION_TIMEOUT = 20 * 60  # 20 mins


@base.ReleaseTracks(base.ReleaseTrack.GA)
@dm_base.UseDmApi(dm_base.DmApiVersion.V2)
class Update(base.UpdateCommand, dm_base.DmCommand):
  """Update a deployment based on a provided config file.

  This command will update a deployment with the new config file provided.
  Different policies for create, update, and delete policies can be specified.
  """

  detailed_help = {
      'EXAMPLES': """
To update an existing deployment with a new config YAML file, run:

  $ {command} my-deployment --config=new_config.yaml

To update an existing deployment with a new config template file, run:

  $ {command} my-deployment --template=new_config.{jinja|py}

To update an existing deployment with a composite type as a new config, run:

  $ {command} my-deployment --composite-type=<project-id>/composite:<new-config>


To preview an update to an existing deployment without actually modifying the resources, run:

  $ {command} my-deployment --config=new_config.yaml --preview

To apply an update that has been previewed, provide the name of the previewed deployment, and no config file:

  $ {command} my-deployment

To specify different create, update, or delete policies, include any subset of the following flags:

  $ {command} my-deployment --config=new_config.yaml --create-policy=acquire --delete-policy=abandon

To perform an update without waiting for the operation to complete, run:

  $ {command} my-deployment --config=new_config.yaml --async

To update an existing deployment with a new config file and a fingerprint, run:

  $ {command} my-deployment --config=new_config.yaml --fingerprint=deployment-fingerprint

Either the `--config`, `--template`, or `--composite-type` flag is required unless launching an already-previewed update to a deployment. If you want to update a deployment's metadata, such as the labels or description, you must run a separate command with `--update-labels`, `--remove-labels`, or `--description`, as applicable.

More information is available at https://cloud.google.com/deployment-manager/docs/deployments/updating-deployments.
""",
  }

  _delete_policy_flag_map = flags.GetDeleteFlagEnumMap(
      (apis.GetMessagesModule('deploymentmanager', 'v2')
       .DeploymentmanagerDeploymentsUpdateRequest.DeletePolicyValueValuesEnum))

  _create_policy_flag_map = arg_utils.ChoiceEnumMapper(
      '--create-policy',
      (apis.GetMessagesModule('deploymentmanager', 'v2')
       .DeploymentmanagerDeploymentsUpdateRequest.CreatePolicyValueValuesEnum),
      help_str='Create policy for resources that have changed in the update',
      default='create-or-acquire')

  _create_policy_v2beta_flag_map = arg_utils.ChoiceEnumMapper(
      '--create-policy',
      (apis.GetMessagesModule('deploymentmanager', 'v2beta')
       .DeploymentmanagerDeploymentsUpdateRequest.CreatePolicyValueValuesEnum),
      help_str='Create policy for resources that have changed in the update',
      default='create-or-acquire')

  @staticmethod
  def Args(parser, version=base.ReleaseTrack.GA):
    """Args is called by calliope to gather arguments for this command.

    Args:
      parser: An argparse parser that you can use to add arguments that go
          on the command line after this command. Positional arguments are
          allowed.
      version: The version this tool is running as. base.ReleaseTrack.GA
          is the default.
    """
    flags.AddDeploymentNameFlag(parser)
    flags.AddPropertiesFlag(parser)
    flags.AddAsyncFlag(parser)

    parser.add_argument(
        '--description',
        help='The new description of the deployment.',
        dest='description'
    )

    group = parser.add_mutually_exclusive_group()
    flags.AddConfigFlags(group)

    if version in [base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA]:
      group.add_argument(
          '--manifest-id',
          help='Manifest Id of a previous deployment. '
          'This flag cannot be used with --config.',
          dest='manifest_id')

    labels_util.AddUpdateLabelsFlags(parser, enable_clear=False)

    parser.add_argument(
        '--preview',
        help='Preview the requested update without making any changes to the '
        'underlying resources. (default=False)',
        dest='preview',
        default=False,
        action='store_true')

    if version in [base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA]:
      Update._create_policy_v2beta_flag_map.choice_arg.AddToParser(parser)
    else:
      Update._create_policy_flag_map.choice_arg.AddToParser(parser)

    Update._delete_policy_flag_map.choice_arg.AddToParser(parser)
    flags.AddFingerprintFlag(parser)

    parser.display_info.AddFormat(flags.RESOURCES_AND_OUTPUTS_FORMAT)

  def Epilog(self, resources_were_displayed):
    """Called after resources are displayed if the default format was used.

    Args:
      resources_were_displayed: True if resources were displayed.
    """
    if not resources_were_displayed:
      log.status.Print('No resources or outputs found in your deployment.')

  def Run(self, args):
    """Run 'deployments update'.

    Args:
      args: argparse.Namespace, The arguments that this command was invoked
          with.

    Returns:
      If --async=true, returns Operation to poll.
      Else, returns a struct containing the list of resources and list of
        outputs in the deployment.

    Raises:
      HttpException: An http error response was received while executing api
          request.
    """
    deployment_ref = self.resources.Parse(
        args.deployment_name,
        params={'project': properties.VALUES.core.project.GetOrFail},
        collection='deploymentmanager.deployments')
    if not args.IsSpecified('format') and args.async_:
      args.format = flags.OPERATION_FORMAT

    patch_request = False
    deployment = self.messages.Deployment(
        name=deployment_ref.deployment,
    )

    if not (args.config is None and args.template is None
            and args.composite_type is None):
      deployment.target = importer.BuildTargetConfig(
          self.messages,
          config=args.config,
          template=args.template,
          composite_type=args.composite_type,
          properties=args.properties)
    elif (self.ReleaseTrack() in [base.ReleaseTrack.ALPHA,
                                  base.ReleaseTrack.BETA]
          and args.manifest_id):
      deployment.target = importer.BuildTargetConfigFromManifest(
          self.client, self.messages,
          dm_base.GetProject(),
          deployment_ref.deployment, args.manifest_id, args.properties)
    # Get the fingerprint from the deployment to update.
    try:
      current_deployment = self.client.deployments.Get(
          self.messages.DeploymentmanagerDeploymentsGetRequest(
              project=dm_base.GetProject(),
              deployment=deployment_ref.deployment
          )
      )

      if args.fingerprint:
        deployment.fingerprint = dm_util.DecodeFingerprint(args.fingerprint)
      else:
        # If no fingerprint is present, default to an empty fingerprint.
        # TODO(b/34966984): Remove the empty default after cleaning up all
        # deployments that has no fingerprint
        deployment.fingerprint = current_deployment.fingerprint or b''

      # Get the credential from the deployment to update.
      if self.ReleaseTrack() in [base.ReleaseTrack.ALPHA] and args.credential:
        deployment.credential = dm_util.CredentialFrom(self.messages,
                                                       args.credential)

      # Update the labels of the deployment

      deployment.labels = self._GetUpdatedDeploymentLabels(
          args, current_deployment)
      # If no config or manifest_id are specified, but try to update labels,
      # only add patch_request header when directly updating a non-previewed
      # deployment

      no_manifest = (self.ReleaseTrack() is
                     base.ReleaseTrack.GA) or not args.manifest_id
      patch_request = not args.config and no_manifest and (
          bool(args.update_labels) or bool(args.remove_labels))
      if args.description is None:
        deployment.description = current_deployment.description
      elif not args.description or args.description.isspace():
        deployment.description = None
      else:
        deployment.description = args.description
    except apitools_exceptions.HttpError as error:
      raise exceptions.HttpException(error, dm_api_util.HTTP_ERROR_FORMAT)

    if patch_request:
      args.format = flags.DEPLOYMENT_FORMAT
    try:
      # Necessary to handle API Version abstraction below
      parsed_delete_flag = Update._delete_policy_flag_map.GetEnumForChoice(
          args.delete_policy).name
      if self.ReleaseTrack() in [
          base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA
      ]:
        parsed_create_flag = (
            Update._create_policy_v2beta_flag_map.GetEnumForChoice(
                args.create_policy).name)
      else:
        parsed_create_flag = (
            Update._create_policy_flag_map.GetEnumForChoice(
                args.create_policy).name)
      request = self.messages.DeploymentmanagerDeploymentsUpdateRequest(
          deploymentResource=deployment,
          project=dm_base.GetProject(),
          deployment=deployment_ref.deployment,
          preview=args.preview,
          createPolicy=(self.messages.DeploymentmanagerDeploymentsUpdateRequest.
                        CreatePolicyValueValuesEnum(parsed_create_flag)),
          deletePolicy=(self.messages.DeploymentmanagerDeploymentsUpdateRequest.
                        DeletePolicyValueValuesEnum(parsed_delete_flag)))
      client = self.client
      client.additional_http_headers['X-Cloud-DM-Patch'] = str(patch_request)
      operation = client.deployments.Update(request)

      # Fetch and print the latest fingerprint of the deployment.
      updated_deployment = dm_api_util.FetchDeployment(
          self.client, self.messages, dm_base.GetProject(),
          deployment_ref.deployment)
      if patch_request:
        if args.async_:
          log.warning(
              'Updating Deployment metadata is synchronous, --async flag '
              'is ignored.')
        log.status.Print('Update deployment metadata completed successfully.')
        return updated_deployment
      dm_util.PrintFingerprint(updated_deployment.fingerprint)
    except apitools_exceptions.HttpError as error:
      raise exceptions.HttpException(error, dm_api_util.HTTP_ERROR_FORMAT)
    if args.async_:
      return operation
    else:
      op_name = operation.name
      try:
        operation = dm_write.WaitForOperation(
            self.client,
            self.messages,
            op_name,
            'update',
            dm_base.GetProject(),
            timeout=OPERATION_TIMEOUT)
        dm_util.LogOperationStatus(operation, 'Update')
      except apitools_exceptions.HttpError as error:
        raise exceptions.HttpException(error, dm_api_util.HTTP_ERROR_FORMAT)

      return dm_api_util.FetchResourcesAndOutputs(self.client, self.messages,
                                                  dm_base.GetProject(),
                                                  deployment_ref.deployment)

  def _GetUpdatedDeploymentLabels(self, args, deployment):
    update_labels = labels_util.GetUpdateLabelsDictFromArgs(args)
    remove_labels = labels_util.GetRemoveLabelsListFromArgs(args)
    return dm_labels.UpdateLabels(deployment.labels,
                                  self.messages.DeploymentLabelEntry,
                                  update_labels, remove_labels)


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
@dm_base.UseDmApi(dm_base.DmApiVersion.ALPHA)
class UpdateAlpha(Update):
  """Update a deployment based on a provided config file.

  This command will update a deployment with the new config file provided.
  Different policies for create, update, and delete policies can be specified.
  """

  @staticmethod
  def Args(parser):
    Update.Args(parser, version=base.ReleaseTrack.ALPHA)
    alpha_flags.AddCredentialFlag(parser)
    parser.display_info.AddFormat(alpha_flags.RESOURCES_AND_OUTPUTS_FORMAT)


@base.ReleaseTracks(base.ReleaseTrack.BETA)
@dm_base.UseDmApi(dm_base.DmApiVersion.V2BETA)
class UpdateBeta(Update):
  """Update a deployment based on a provided config file.

  This command will update a deployment with the new config file provided.
  Different policies for create, update, and delete policies can be specified.
  """

  @staticmethod
  def Args(parser):
    Update.Args(parser, version=base.ReleaseTrack.BETA)



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Resource definitions for Cloud Platform APIs generated from gapic."""

import enum


BASE_URL = 'https://run.googleapis.com/v2/'
DOCS_URL = 'https://cloud.google.com/apis/docs/overview'


class Collections(enum.Enum):
  """Collections for all supported apis."""

  PROJECTS = (
      'projects',
      'projects/{project}',
      {},
      ['project'],
      True
  )
  PROJECTS_LOCATIONS = (
      'projects.locations',
      'projects/{project}/locations/{location}',
      {},
      ['project', 'location'],
      True
  )
  PROJECTS_LOCATIONS_INSTANCES = (
      'projects.locations.instances',
      'projects/{project}/locations/{location}/instances/{instance}',
      {},
      ['project', 'location', 'instance'],
      True
  )
  PROJECTS_LOCATIONS_JOBS = (
      'projects.locations.jobs',
      'projects/{project}/locations/{location}/jobs/{job}',
      {},
      ['project', 'location', 'job'],
      True
  )
  PROJECTS_LOCATIONS_JOBS_EXECUTIONS = (
      'projects.locations.jobs.executions',
      'projects/{project}/locations/{location}/jobs/{job}/executions/'
      '{execution}',
      {},
      ['project', 'location', 'job', 'execution'],
      True
  )
  PROJECTS_LOCATIONS_JOBS_EXECUTIONS_TASKS = (
      'projects.locations.jobs.executions.tasks',
      'projects/{project}/locations/{location}/jobs/{job}/executions/'
      '{execution}/tasks/{task}',
      {},
      ['project', 'location', 'job', 'execution', 'task'],
      True
  )
  PROJECTS_LOCATIONS_SERVICES = (
      'projects.locations.services',
      'projects/{project}/locations/{location}/services/{service}',
      {},
      ['project', 'location', 'service'],
      True
  )
  PROJECTS_LOCATIONS_SERVICES_REVISIONS = (
      'projects.locations.services.revisions',
      'projects/{project}/locations/{location}/services/{service}/revisions/'
      '{revision}',
      {},
      ['project', 'location', 'service', 'revision'],
      True
  )
  PROJECTS_LOCATIONS_WORKERPOOLS = (
      'projects.locations.workerPools',
      'projects/{project}/locations/{location}/workerPools/{worker_pool}',
      {},
      ['project', 'location', 'worker_pool'],
      True
  )

  def __init__(self, collection_name, path, flat_paths, params,
               enable_uri_parsing):
    self.collection_name = collection_name
    self.path = path
    self.flat_paths = flat_paths
    self.params = params
    self.enable_uri_parsing = enable_uri_parsing



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The command to update Service Mesh Feature."""


from apitools.base.py import exceptions as apitools_exceptions
from googlecloudsdk.api_lib.container.fleet import util
from googlecloudsdk.calliope import actions
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.anthos.common import file_parsers
from googlecloudsdk.command_lib.container.fleet import resources
from googlecloudsdk.command_lib.container.fleet.features import base as features_base
from googlecloudsdk.command_lib.container.fleet.membershipfeatures import base as mf_base
from googlecloudsdk.command_lib.container.fleet.mesh import utils


# LINT.IfChange(update_v2)
def _RunUpdateV2(cmd, args):
  """Runs the update command implementation that is common across release tracks.

  For membership level spec update, we will use v2alpha API to directly update
  the membership feature resource.

  Args:
    cmd: the release track specific command
    args: the args passed to the command
  """

  is_fleet_update = args.fleet_default_member_config or getattr(
      args, 'modernization_compatibility', None
  )
  memberships = []
  resource = False
  update_mask = []

  # Deprecated non-resource arg
  if args.IsKnownAndSpecified('membership'):
    resource = False
    memberships = utils.ParseMemberships(args)
  elif not is_fleet_update:
    resource = True
    memberships = features_base.ParseMembershipsPlural(
        args, prompt=True, search=True
    )

  for membership in memberships:
    if not resource:
      membership = cmd.MembershipResourceName(membership)

    patch_v2 = cmd.messages_v2.FeatureSpec()

    try:
      existing_membership_feature = cmd.GetMembershipFeature(membership)
    except apitools_exceptions.HttpNotFoundError:
      existing_membership_feature = cmd.messages_v2.MembershipFeature()

    if existing_membership_feature.spec:
      patch_v2 = existing_membership_feature.spec

    if not patch_v2.servicemesh:
      patch_v2.servicemesh = cmd.messages_v2.ServiceMeshSpec()

    if hasattr(args, 'origin') and args.origin is not None:
      if args.origin == 'fleet':
        patch_v2.origin = cmd.messages_v2.Origin(
            type=cmd.messages_v2.Origin.TypeValueValuesEnum('FLEET')
        )

    if hasattr(args, 'management') and args.management is not None:
      management_v2 = (
          cmd.messages_v2.ServiceMeshSpec.ManagementValueValuesEnum(
              'MANAGEMENT_MANUAL'))
      if args.management == 'automatic':
        management_v2 = (
            cmd.messages_v2.ServiceMeshSpec.ManagementValueValuesEnum(
                'MANAGEMENT_AUTOMATIC'))
      if args.management == 'not-installed':
        management_v2 = (
            cmd.messages_v2.ServiceMeshSpec.ManagementValueValuesEnum(
                'MANAGEMENT_NOT_INSTALLED'))
      patch_v2.servicemesh.management = management_v2

    if args.control_plane is not None:
      control_plane_v2 = (
          cmd.messages_v2.ServiceMeshSpec.ControlPlaneValueValuesEnum(
              'MANUAL'))
      if args.control_plane == 'automatic':
        control_plane_v2 = (
            cmd.messages_v2.ServiceMeshSpec.ControlPlaneValueValuesEnum(
                'AUTOMATIC'))
      elif args.control_plane == 'unspecified':
        control_plane_v2 = (
            cmd.messages_v2.ServiceMeshSpec.ControlPlaneValueValuesEnum(
                'CONTROL_PLANE_MANAGEMENT_UNSPECIFIED'))
      patch_v2.servicemesh.controlPlane = control_plane_v2

    if hasattr(args, 'config_api') and args.config_api is not None:
      config_api = (
          cmd.messages_v2.ServiceMeshSpec.ConfigApiValueValuesEnum(
              'CONFIG_API_UNSPECIFIED'
          )
      )
      if args.config_api == 'istio':
        config_api = (
            cmd.messages_v2.ServiceMeshSpec.ConfigApiValueValuesEnum(
                'CONFIG_API_ISTIO'
            )
        )
      if args.config_api == 'gateway':
        config_api = (
            cmd.messages_v2.ServiceMeshSpec.ConfigApiValueValuesEnum(
                'CONFIG_API_GATEWAY'
            )
        )
      patch_v2.servicemesh.configApi = config_api

    cmd.UpdateV2(
        membership, ['spec'], cmd.messages_v2.MembershipFeature(spec=patch_v2)
    )

  f = cmd.messages.Feature()

  if args.fleet_default_member_config:
    # Load config YAML file.
    loaded_config = file_parsers.YamlConfigFile(
        file_path=args.fleet_default_member_config,
        item_type=utils.FleetDefaultMemberConfigObject,
    )

    # Create new service mesh feature spec.
    member_config = utils.ParseFleetDefaultMemberConfigV2(
        loaded_config, cmd.messages
    )
    f.fleetDefaultMemberConfig = (
        cmd.messages.CommonFleetDefaultMemberConfigSpec(mesh=member_config)
    )
    update_mask.append('fleet_default_member_config')
  elif getattr(args, 'modernization_compatibility', None):
    spec = cmd.messages.ServiceMeshFeatureSpec()
    mod_compat_enum = (
        cmd.messages.ServiceMeshFeatureSpec
        .ModernizationCompatibilityValueValuesEnum
    )
    if args.modernization_compatibility == 'validation-enabled':
      spec.modernizationCompatibility = mod_compat_enum.VALIDATION_ENABLED
    elif args.modernization_compatibility == 'validation-disabled':
      spec.modernizationCompatibility = mod_compat_enum.VALIDATION_DISABLED
    f.spec = cmd.messages.CommonFeatureSpec(mesh=spec)
    update_mask.append('spec.mesh.modernizationCompatibility')

  if update_mask:
    cmd.Update(update_mask, f)
# LINT.ThenChange(:update_v1)


# LINT.IfChange(update_v1)
def _RunUpdate(cmd, args):
  """Runs the update command implementation that is common across release tracks.

  Args:
    cmd: the release track specific command
    args: the args passed to the command
  """

  is_fleet_update = args.fleet_default_member_config or getattr(
      args, 'modernization_compatibility', None
  )
  memberships = []
  resource = False
  update_mask = []

  # Deprecated non-resource arg
  if args.IsKnownAndSpecified('membership'):
    resource = False
    memberships = utils.ParseMemberships(args)
    update_mask = ['membershipSpecs']
  elif not is_fleet_update:
    resource = True
    memberships = features_base.ParseMembershipsPlural(
        args, prompt=True, search=True
    )
    update_mask = ['membershipSpecs']

  f = cmd.GetFeature()
  membership_specs = {}
  for membership in memberships:
    if not resource:
      membership = cmd.MembershipResourceName(membership)
    patch = cmd.messages.MembershipFeatureSpec()

    for name, spec in cmd.hubclient.ToPyDict(f.membershipSpecs).items():
      if (
          util.MembershipShortname(name) == util.MembershipShortname(membership)
          and spec
      ):
        patch = spec
    if not patch.mesh:
      patch.mesh = cmd.messages.ServiceMeshMembershipSpec()

    if hasattr(args, 'origin') and args.origin is not None:
      if args.origin == 'fleet':
        patch.origin = cmd.messages.Origin(
            type=cmd.messages.Origin.TypeValueValuesEnum('FLEET')
        )

    if hasattr(args, 'management') and args.management is not None:
      management = (
          cmd.messages.ServiceMeshMembershipSpec.ManagementValueValuesEnum(
              'MANAGEMENT_MANUAL'
          )
      )
      if args.management == 'automatic':
        management = (
            cmd.messages.ServiceMeshMembershipSpec.ManagementValueValuesEnum(
                'MANAGEMENT_AUTOMATIC'
            )
        )
      if args.management == 'not-installed':
        management = (
            cmd.messages_v2.ServiceMeshSpec.ManagementValueValuesEnum(
                'MANAGEMENT_NOT_INSTALLED'))
      patch.mesh.management = management

    if args.control_plane is not None:
      control_plane = (
          cmd.messages.ServiceMeshMembershipSpec.ControlPlaneValueValuesEnum(
              'MANUAL'
          )
      )
      if args.control_plane == 'automatic':
        control_plane = (
            cmd.messages.ServiceMeshMembershipSpec.ControlPlaneValueValuesEnum(
                'AUTOMATIC'
            )
        )
      elif args.control_plane == 'unspecified':
        control_plane = (
            cmd.messages.ServiceMeshMembershipSpec.ControlPlaneValueValuesEnum(
                'CONTROL_PLANE_MANAGEMENT_UNSPECIFIED'
            )
        )
      patch.mesh.controlPlane = control_plane

    if hasattr(args, 'config_api') and args.config_api is not None:
      config_api = (
          cmd.messages.ServiceMeshMembershipSpec.ConfigApiValueValuesEnum(
              'CONFIG_API_UNSPECIFIED'
          )
      )
      if args.config_api == 'istio':
        config_api = (
            cmd.messages.ServiceMeshMembershipSpec.ConfigApiValueValuesEnum(
                'CONFIG_API_ISTIO'
            )
        )
      if args.config_api == 'gateway':
        config_api = (
            cmd.messages.ServiceMeshMembershipSpec.ConfigApiValueValuesEnum(
                'CONFIG_API_GATEWAY'
            )
        )
      patch.mesh.configApi = config_api

    membership_specs[membership] = patch

  f = cmd.messages.Feature(
      membershipSpecs=cmd.hubclient.ToMembershipSpecs(membership_specs)
  )

  if args.fleet_default_member_config:
    # Load config YAML file.
    loaded_config = file_parsers.YamlConfigFile(
        file_path=args.fleet_default_member_config,
        item_type=utils.FleetDefaultMemberConfigObject,
    )

    # Create new service mesh feature spec.
    member_config = utils.ParseFleetDefaultMemberConfig(
        loaded_config, cmd.messages
    )
    f.fleetDefaultMemberConfig = (
        cmd.messages.CommonFleetDefaultMemberConfigSpec(mesh=member_config)
    )
    update_mask.append('fleet_default_member_config')
  elif getattr(args, 'modernization_compatibility', None):
    spec = cmd.messages.ServiceMeshFeatureSpec()
    mod_compat_enum = (
        cmd.messages.ServiceMeshFeatureSpec
        .ModernizationCompatibilityValueValuesEnum
    )
    if args.modernization_compatibility == 'validation-enabled':
      spec.modernizationCompatibility = mod_compat_enum.VALIDATION_ENABLED
    elif args.modernization_compatibility == 'validation-disabled':
      spec.modernizationCompatibility = mod_compat_enum.VALIDATION_DISABLED
    f.spec = cmd.messages.CommonFeatureSpec(mesh=spec)
    update_mask.append('spec.mesh.modernizationCompatibility')

  cmd.Update(update_mask, f)
# LINT.ThenChange(:update_v2)


@base.DefaultUniverseOnly
@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class UpdateAlpha(features_base.UpdateCommand, mf_base.UpdateCommand):
  """Update the configuration of the Service Mesh Feature.

  Update the Service Mesh Feature Spec of a membership.

  ## EXAMPLES

  To update the control plane management of comma separated memberships like
  `MEMBERSHIP1,MEMBERSHIP2`, run:

    $ {command} --memberships=MEMBERSHIP1,MEMBERSHIP2
      --control-plane=CONTROL_PLANE
  """

  feature_name = 'servicemesh'
  mf_name = 'servicemesh'

  @staticmethod
  def Args(parser):
    args_group = parser.add_mutually_exclusive_group(required=True)

    args_group.add_argument(
        '--fleet-default-member-config',
        type=str,
        help="""The path to a service-mesh.yaml configuration file.

        To enable the Service Mesh Feature with a fleet-level default
        membership configuration, run:

          $ {command} --fleet-default-member-config=/path/to/service-mesh.yaml""",
    )
    args_group.add_argument(
        '--modernization-compatibility',
        choices=['validation-enabled', 'validation-disabled'],
        help="""The modernization compatibility of the mesh.

        To set the modernization compatibility of the mesh, run:

          $ {command} --modernization-compatibility=validation-enabled""",
    )

    membership_group = args_group.add_group(
        'Component options',
    )

    membership_names_group = membership_group.add_mutually_exclusive_group()
    resources.AddMembershipResourceArg(
        membership_names_group,
        plural=True,
        membership_help=(
            'Membership names to update, separated by commas if '
            'multiple are supplied.'
        ),
    )
    membership_names_group.add_argument(
        '--membership',
        type=str,
        help='Membership name to update.',
        action=actions.DeprecationAction(
            '--membership',
            warn=(
                'The {flag_name} flag is now '
                'deprecated. Please use `--memberships` '
                'instead.'
            ),
        ),
    )

    membership_config_group = membership_group.add_group(required=True)
    membership_config_group.add_argument(
        '--origin',
        choices=['fleet'],
        help='Changing the origin of the membership.',
    )
    membership_controlplane_group = (
        membership_config_group.add_mutually_exclusive_group()
    )

    membership_controlplane_group.add_argument(
        '--config-api',
        choices=['istio', 'gateway'],
        help='The API to use for mesh configuration.',
    )
    membership_controlplane_group.add_argument(
        '--management',
        choices=['automatic', 'manual', 'not-installed'],
        help='The management mode to update to.',
    )
    membership_controlplane_group.add_argument(
        '--control-plane',
        choices=['automatic', 'manual', 'unspecified'],
        help='Control plane management to update to.',
        action=actions.DeprecationAction(
            '--control-plane',
            warn=(
                'The {flag_name} flag is now '
                'deprecated. Please use `--management` '
                'instead. '
                'See https://cloud.google.com/service-mesh/docs/managed/provision-managed-anthos-service-mesh'
            ),
        ),
    )

  def Run(self, args):
    # If the user is using the fleet default config, we will still use the v1
    # Feature API for the update.
    use_fleet_default_config = (
        hasattr(args, 'origin')
        and args.origin == 'fleet'
    )
    if not use_fleet_default_config:
      _RunUpdateV2(self, args)
    else:
      _RunUpdate(self, args)


@base.DefaultUniverseOnly
@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.GA)
class UpdateGA(features_base.UpdateCommand, mf_base.UpdateCommand):
  """Update the configuration of the Service Mesh Feature.

  Update the Service Mesh Feature Spec of a Membership.

  ## EXAMPLES

  To update the control plane management of comma separated Memberships like
  `membership1,membership2`, run:

    $ {command} --memberships=membership1,membership2
      --control-plane=CONTROL_PLANE
  """

  feature_name = 'servicemesh'
  mf_name = 'servicemesh'

  @staticmethod
  def Args(parser):
    args_group = parser.add_mutually_exclusive_group(required=True)

    args_group.add_argument(
        '--fleet-default-member-config',
        type=str,
        help="""The path to a service-mesh.yaml configuration file.

        To enable the Service Mesh Feature with a fleet-level default
        membership configuration, run:

          $ {command} --fleet-default-member-config=/path/to/service-mesh.yaml""",
    )

    membership_group = args_group.add_group()

    membership_names_group = membership_group.add_mutually_exclusive_group()
    resources.AddMembershipResourceArg(
        membership_names_group,
        plural=True,
        membership_help=(
            'Membership names to update, separated by commas if '
            'multiple are supplied.'
        ),
    )

    membership_configs_group = membership_group.add_group(required=True)
    membership_configs_group.add_argument(
        '--origin',
        choices=['fleet'],
        help='Changing the origin of the membership.',
    )
    membership_controlplane_group = (
        membership_configs_group.add_mutually_exclusive_group()
    )

    membership_controlplane_group.add_argument(
        '--config-api',
        choices=['istio', 'gateway'],
        help='The API to use for mesh configuration.',
    )
    membership_controlplane_group.add_argument(
        '--management',
        choices=['automatic', 'manual'],
        help='The management mode to update to.',
    )
    membership_controlplane_group.add_argument(
        '--control-plane',
        choices=['automatic', 'manual', 'unspecified'],
        help='Control plane management to update to.',
        action=actions.DeprecationAction(
            '--control-plane',
            warn=(
                'The {flag_name} flag is now '
                'deprecated. Please use `--management` '
                'instead. '
                'See https://cloud.google.com/service-mesh/docs/managed/provision-managed-anthos-service-mesh'
            ),
        ),
    )

  def Run(self, args):
    # If the user is using the fleet default config, we will still use the v1
    # Feature API for the update.
    use_fleet_default_config = (
        hasattr(args, 'origin')
        and args.origin == 'fleet'
    )
    if not use_fleet_default_config:
      _RunUpdateV2(self, args)
    else:
      _RunUpdate(self, args)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

import proto  # type: ignore

from cloudsdk.google.protobuf import timestamp_pb2  # type: ignore
from googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types import api_auth as gca_api_auth
from googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types import encryption_spec as gca_encryption_spec
from googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types import io


__protobuf__ = proto.module(
    package='google.cloud.aiplatform.v1',
    manifest={
        'RagEmbeddingModelConfig',
        'RagVectorDbConfig',
        'FileStatus',
        'VertexAiSearchConfig',
        'CorpusStatus',
        'RagCorpus',
        'RagFile',
        'RagChunk',
        'RagFileChunkingConfig',
        'RagFileTransformationConfig',
        'RagFileParsingConfig',
        'UploadRagFileConfig',
        'ImportRagFilesConfig',
        'RagManagedDbConfig',
        'RagEngineConfig',
    },
)


class RagEmbeddingModelConfig(proto.Message):
    r"""Config for the embedding model to use for RAG.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        vertex_prediction_endpoint (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.RagEmbeddingModelConfig.VertexPredictionEndpoint):
            The Vertex AI Prediction Endpoint that either
            refers to a publisher model or an endpoint that
            is hosting a 1P fine-tuned text embedding model.
            Endpoints hosting non-1P fine-tuned text
            embedding models are currently not supported.
            This is used for dense vector search.

            This field is a member of `oneof`_ ``model_config``.
    """

    class VertexPredictionEndpoint(proto.Message):
        r"""Config representing a model hosted on Vertex Prediction
        Endpoint.

        Attributes:
            endpoint (str):
                Required. The endpoint resource name. Format:
                ``projects/{project}/locations/{location}/publishers/{publisher}/models/{model}``
                or
                ``projects/{project}/locations/{location}/endpoints/{endpoint}``
            model (str):
                Output only. The resource name of the model that is deployed
                on the endpoint. Present only when the endpoint is not a
                publisher model. Pattern:
                ``projects/{project}/locations/{location}/models/{model}``
            model_version_id (str):
                Output only. Version ID of the model that is
                deployed on the endpoint. Present only when the
                endpoint is not a publisher model.
        """

        endpoint: str = proto.Field(
            proto.STRING,
            number=1,
        )
        model: str = proto.Field(
            proto.STRING,
            number=2,
        )
        model_version_id: str = proto.Field(
            proto.STRING,
            number=3,
        )

    vertex_prediction_endpoint: VertexPredictionEndpoint = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof='model_config',
        message=VertexPredictionEndpoint,
    )


class RagVectorDbConfig(proto.Message):
    r"""Config for the Vector DB to use for RAG.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        rag_managed_db (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.RagVectorDbConfig.RagManagedDb):
            The config for the RAG-managed Vector DB.

            This field is a member of `oneof`_ ``vector_db``.
        pinecone (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.RagVectorDbConfig.Pinecone):
            The config for the Pinecone.

            This field is a member of `oneof`_ ``vector_db``.
        vertex_vector_search (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.RagVectorDbConfig.VertexVectorSearch):
            The config for the Vertex Vector Search.

            This field is a member of `oneof`_ ``vector_db``.
        api_auth (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.ApiAuth):
            Authentication config for the chosen Vector
            DB.
        rag_embedding_model_config (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.RagEmbeddingModelConfig):
            Optional. Immutable. The embedding model
            config of the Vector DB.
    """

    class RagManagedDb(proto.Message):
        r"""The config for the default RAG-managed Vector DB.

        This message has `oneof`_ fields (mutually exclusive fields).
        For each oneof, at most one member field can be set at the same time.
        Setting any member of the oneof automatically clears all other
        members.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            knn (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.RagVectorDbConfig.RagManagedDb.KNN):
                Performs a KNN search on RagCorpus.
                Default choice if not specified.

                This field is a member of `oneof`_ ``retrieval_strategy``.
            ann (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.RagVectorDbConfig.RagManagedDb.ANN):
                Performs an ANN search on RagCorpus. Use this
                if you have a lot of files (> 10K) in your
                RagCorpus and want to reduce the search latency.

                This field is a member of `oneof`_ ``retrieval_strategy``.
        """

        class KNN(proto.Message):
            r"""Config for KNN search.
            """

        class ANN(proto.Message):
            r"""Config for ANN search.

            RagManagedDb uses a tree-based structure to partition data and
            facilitate faster searches. As a tradeoff, it requires longer
            indexing time and manual triggering of index rebuild via the
            ImportRagFiles and UpdateRagCorpus API.

            Attributes:
                tree_depth (int):
                    The depth of the tree-based structure. Only
                    depth values of 2 and 3 are supported.

                    Recommended value is 2 if you have if you have
                    O(10K) files in the RagCorpus and set this to 3
                    if more than that.

                    Default value is 2.
                leaf_count (int):
                    Number of leaf nodes in the tree-based structure. Each leaf
                    node contains groups of closely related vectors along with
                    their corresponding centroid.

                    Recommended value is 10 \* sqrt(num of RagFiles in your
                    RagCorpus).

                    Default value is 500.
            """

            tree_depth: int = proto.Field(
                proto.INT32,
                number=1,
            )
            leaf_count: int = proto.Field(
                proto.INT32,
                number=2,
            )

        knn: 'RagVectorDbConfig.RagManagedDb.KNN' = proto.Field(
            proto.MESSAGE,
            number=1,
            oneof='retrieval_strategy',
            message='RagVectorDbConfig.RagManagedDb.KNN',
        )
        ann: 'RagVectorDbConfig.RagManagedDb.ANN' = proto.Field(
            proto.MESSAGE,
            number=2,
            oneof='retrieval_strategy',
            message='RagVectorDbConfig.RagManagedDb.ANN',
        )

    class Pinecone(proto.Message):
        r"""The config for the Pinecone.

        Attributes:
            index_name (str):
                Pinecone index name.
                This value cannot be changed after it's set.
        """

        index_name: str = proto.Field(
            proto.STRING,
            number=1,
        )

    class VertexVectorSearch(proto.Message):
        r"""The config for the Vertex Vector Search.

        Attributes:
            index_endpoint (str):
                The resource name of the Index Endpoint. Format:
                ``projects/{project}/locations/{location}/indexEndpoints/{index_endpoint}``
            index (str):
                The resource name of the Index. Format:
                ``projects/{project}/locations/{location}/indexes/{index}``
        """

        index_endpoint: str = proto.Field(
            proto.STRING,
            number=1,
        )
        index: str = proto.Field(
            proto.STRING,
            number=2,
        )

    rag_managed_db: RagManagedDb = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof='vector_db',
        message=RagManagedDb,
    )
    pinecone: Pinecone = proto.Field(
        proto.MESSAGE,
        number=3,
        oneof='vector_db',
        message=Pinecone,
    )
    vertex_vector_search: VertexVectorSearch = proto.Field(
        proto.MESSAGE,
        number=6,
        oneof='vector_db',
        message=VertexVectorSearch,
    )
    api_auth: gca_api_auth.ApiAuth = proto.Field(
        proto.MESSAGE,
        number=5,
        message=gca_api_auth.ApiAuth,
    )
    rag_embedding_model_config: 'RagEmbeddingModelConfig' = proto.Field(
        proto.MESSAGE,
        number=7,
        message='RagEmbeddingModelConfig',
    )


class FileStatus(proto.Message):
    r"""RagFile status.

    Attributes:
        state (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.FileStatus.State):
            Output only. RagFile state.
        error_status (str):
            Output only. Only when the ``state`` field is ERROR.
    """
    class State(proto.Enum):
        r"""RagFile state.

        Values:
            STATE_UNSPECIFIED (0):
                RagFile state is unspecified.
            ACTIVE (1):
                RagFile resource has been created and indexed
                successfully.
            ERROR (2):
                RagFile resource is in a problematic state. See
                ``error_message`` field for details.
        """
        STATE_UNSPECIFIED = 0
        ACTIVE = 1
        ERROR = 2

    state: State = proto.Field(
        proto.ENUM,
        number=1,
        enum=State,
    )
    error_status: str = proto.Field(
        proto.STRING,
        number=2,
    )


class VertexAiSearchConfig(proto.Message):
    r"""Config for the Vertex AI Search.

    Attributes:
        serving_config (str):
            Vertex AI Search Serving Config resource full name. For
            example,
            ``projects/{project}/locations/{location}/collections/{collection}/engines/{engine}/servingConfigs/{serving_config}``
            or
            ``projects/{project}/locations/{location}/collections/{collection}/dataStores/{data_store}/servingConfigs/{serving_config}``.
    """

    serving_config: str = proto.Field(
        proto.STRING,
        number=1,
    )


class CorpusStatus(proto.Message):
    r"""RagCorpus status.

    Attributes:
        state (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.CorpusStatus.State):
            Output only. RagCorpus life state.
        error_status (str):
            Output only. Only when the ``state`` field is ERROR.
    """
    class State(proto.Enum):
        r"""RagCorpus life state.

        Values:
            UNKNOWN (0):
                This state is not supposed to happen.
            INITIALIZED (1):
                RagCorpus resource entry is initialized, but
                hasn't done validation.
            ACTIVE (2):
                RagCorpus is provisioned successfully and is
                ready to serve.
            ERROR (3):
                RagCorpus is in a problematic situation. See
                ``error_message`` field for details.
        """
        UNKNOWN = 0
        INITIALIZED = 1
        ACTIVE = 2
        ERROR = 3

    state: State = proto.Field(
        proto.ENUM,
        number=1,
        enum=State,
    )
    error_status: str = proto.Field(
        proto.STRING,
        number=2,
    )


class RagCorpus(proto.Message):
    r"""A RagCorpus is a RagFile container and a project can have
    multiple RagCorpora.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        vector_db_config (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.RagVectorDbConfig):
            Optional. Immutable. The config for the
            Vector DBs.

            This field is a member of `oneof`_ ``backend_config``.
        vertex_ai_search_config (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.VertexAiSearchConfig):
            Optional. Immutable. The config for the
            Vertex AI Search.

            This field is a member of `oneof`_ ``backend_config``.
        name (str):
            Output only. The resource name of the
            RagCorpus.
        display_name (str):
            Required. The display name of the RagCorpus.
            The name can be up to 128 characters long and
            can consist of any UTF-8 characters.
        description (str):
            Optional. The description of the RagCorpus.
        create_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. Timestamp when this RagCorpus
            was created.
        update_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. Timestamp when this RagCorpus
            was last updated.
        corpus_status (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.CorpusStatus):
            Output only. RagCorpus state.
        encryption_spec (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.EncryptionSpec):
            Optional. Immutable. The CMEK key name used
            to encrypt at-rest data related to this Corpus.
            Only applicable to RagManagedDb option for
            Vector DB. This field can only be set at corpus
            creation time, and cannot be updated or deleted.
        satisfies_pzs (bool):
            Output only. Reserved for future use.
        satisfies_pzi (bool):
            Output only. Reserved for future use.
    """

    vector_db_config: 'RagVectorDbConfig' = proto.Field(
        proto.MESSAGE,
        number=9,
        oneof='backend_config',
        message='RagVectorDbConfig',
    )
    vertex_ai_search_config: 'VertexAiSearchConfig' = proto.Field(
        proto.MESSAGE,
        number=10,
        oneof='backend_config',
        message='VertexAiSearchConfig',
    )
    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    display_name: str = proto.Field(
        proto.STRING,
        number=2,
    )
    description: str = proto.Field(
        proto.STRING,
        number=3,
    )
    create_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=4,
        message=timestamp_pb2.Timestamp,
    )
    update_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=5,
        message=timestamp_pb2.Timestamp,
    )
    corpus_status: 'CorpusStatus' = proto.Field(
        proto.MESSAGE,
        number=8,
        message='CorpusStatus',
    )
    encryption_spec: gca_encryption_spec.EncryptionSpec = proto.Field(
        proto.MESSAGE,
        number=12,
        message=gca_encryption_spec.EncryptionSpec,
    )
    satisfies_pzs: bool = proto.Field(
        proto.BOOL,
        number=19,
    )
    satisfies_pzi: bool = proto.Field(
        proto.BOOL,
        number=20,
    )


class RagFile(proto.Message):
    r"""A RagFile contains user data for chunking, embedding and
    indexing.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        gcs_source (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.GcsSource):
            Output only. Google Cloud Storage location of
            the RagFile. It does not support wildcards in
            the Cloud Storage uri for now.

            This field is a member of `oneof`_ ``rag_file_source``.
        google_drive_source (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.GoogleDriveSource):
            Output only. Google Drive location. Supports
            importing individual files as well as Google
            Drive folders.

            This field is a member of `oneof`_ ``rag_file_source``.
        direct_upload_source (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.DirectUploadSource):
            Output only. The RagFile is encapsulated and
            uploaded in the UploadRagFile request.

            This field is a member of `oneof`_ ``rag_file_source``.
        slack_source (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.SlackSource):
            The RagFile is imported from a Slack channel.

            This field is a member of `oneof`_ ``rag_file_source``.
        jira_source (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.JiraSource):
            The RagFile is imported from a Jira query.

            This field is a member of `oneof`_ ``rag_file_source``.
        share_point_sources (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.SharePointSources):
            The RagFile is imported from a SharePoint
            source.

            This field is a member of `oneof`_ ``rag_file_source``.
        name (str):
            Output only. The resource name of the
            RagFile.
        display_name (str):
            Required. The display name of the RagFile.
            The name can be up to 128 characters long and
            can consist of any UTF-8 characters.
        description (str):
            Optional. The description of the RagFile.
        create_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. Timestamp when this RagFile was
            created.
        update_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. Timestamp when this RagFile was
            last updated.
        file_status (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.FileStatus):
            Output only. State of the RagFile.
        user_metadata (str):
            Output only. The metadata for metadata search. The
            user_metadata Needs to be in JSON format.
    """

    gcs_source: io.GcsSource = proto.Field(
        proto.MESSAGE,
        number=8,
        oneof='rag_file_source',
        message=io.GcsSource,
    )
    google_drive_source: io.GoogleDriveSource = proto.Field(
        proto.MESSAGE,
        number=9,
        oneof='rag_file_source',
        message=io.GoogleDriveSource,
    )
    direct_upload_source: io.DirectUploadSource = proto.Field(
        proto.MESSAGE,
        number=10,
        oneof='rag_file_source',
        message=io.DirectUploadSource,
    )
    slack_source: io.SlackSource = proto.Field(
        proto.MESSAGE,
        number=11,
        oneof='rag_file_source',
        message=io.SlackSource,
    )
    jira_source: io.JiraSource = proto.Field(
        proto.MESSAGE,
        number=12,
        oneof='rag_file_source',
        message=io.JiraSource,
    )
    share_point_sources: io.SharePointSources = proto.Field(
        proto.MESSAGE,
        number=14,
        oneof='rag_file_source',
        message=io.SharePointSources,
    )
    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    display_name: str = proto.Field(
        proto.STRING,
        number=2,
    )
    description: str = proto.Field(
        proto.STRING,
        number=3,
    )
    create_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=6,
        message=timestamp_pb2.Timestamp,
    )
    update_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=7,
        message=timestamp_pb2.Timestamp,
    )
    file_status: 'FileStatus' = proto.Field(
        proto.MESSAGE,
        number=13,
        message='FileStatus',
    )
    user_metadata: str = proto.Field(
        proto.STRING,
        number=15,
    )


class RagChunk(proto.Message):
    r"""A RagChunk includes the content of a chunk of a RagFile, and
    associated metadata.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        text (str):
            The content of the chunk.
        page_span (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.RagChunk.PageSpan):
            If populated, represents where the chunk
            starts and ends in the document.

            This field is a member of `oneof`_ ``_page_span``.
    """

    class PageSpan(proto.Message):
        r"""Represents where the chunk starts and ends in the document.

        Attributes:
            first_page (int):
                Page where chunk starts in the document.
                Inclusive. 1-indexed.
            last_page (int):
                Page where chunk ends in the document.
                Inclusive. 1-indexed.
        """

        first_page: int = proto.Field(
            proto.INT32,
            number=1,
        )
        last_page: int = proto.Field(
            proto.INT32,
            number=2,
        )

    text: str = proto.Field(
        proto.STRING,
        number=1,
    )
    page_span: PageSpan = proto.Field(
        proto.MESSAGE,
        number=2,
        optional=True,
        message=PageSpan,
    )


class RagFileChunkingConfig(proto.Message):
    r"""Specifies the size and overlap of chunks for RagFiles.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        fixed_length_chunking (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.RagFileChunkingConfig.FixedLengthChunking):
            Specifies the fixed length chunking config.

            This field is a member of `oneof`_ ``chunking_config``.
    """

    class FixedLengthChunking(proto.Message):
        r"""Specifies the fixed length chunking config.

        Attributes:
            chunk_size (int):
                The size of the chunks.
            chunk_overlap (int):
                The overlap between chunks.
        """

        chunk_size: int = proto.Field(
            proto.INT32,
            number=1,
        )
        chunk_overlap: int = proto.Field(
            proto.INT32,
            number=2,
        )

    fixed_length_chunking: FixedLengthChunking = proto.Field(
        proto.MESSAGE,
        number=3,
        oneof='chunking_config',
        message=FixedLengthChunking,
    )


class RagFileTransformationConfig(proto.Message):
    r"""Specifies the transformation config for RagFiles.

    Attributes:
        rag_file_chunking_config (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.RagFileChunkingConfig):
            Specifies the chunking config for RagFiles.
    """

    rag_file_chunking_config: 'RagFileChunkingConfig' = proto.Field(
        proto.MESSAGE,
        number=1,
        message='RagFileChunkingConfig',
    )


class RagFileParsingConfig(proto.Message):
    r"""Specifies the parsing config for RagFiles.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        layout_parser (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.RagFileParsingConfig.LayoutParser):
            The Layout Parser to use for RagFiles.

            This field is a member of `oneof`_ ``parser``.
        llm_parser (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.RagFileParsingConfig.LlmParser):
            The LLM Parser to use for RagFiles.

            This field is a member of `oneof`_ ``parser``.
    """

    class LayoutParser(proto.Message):
        r"""Document AI Layout Parser config.

        Attributes:
            processor_name (str):
                The full resource name of a Document AI processor or
                processor version. The processor must have type
                ``LAYOUT_PARSER_PROCESSOR``. If specified, the
                ``additional_config.parse_as_scanned_pdf`` field must be
                false. Format:

                - ``projects/{project_id}/locations/{location}/processors/{processor_id}``
                - ``projects/{project_id}/locations/{location}/processors/{processor_id}/processorVersions/{processor_version_id}``
            max_parsing_requests_per_min (int):
                The maximum number of requests the job is
                allowed to make to the Document AI processor per
                minute. Consult
                https://cloud.google.com/document-ai/quotas and
                the Quota page for your project to set an
                appropriate value here. If unspecified, a
                default value of 120 QPM would be used.
        """

        processor_name: str = proto.Field(
            proto.STRING,
            number=1,
        )
        max_parsing_requests_per_min: int = proto.Field(
            proto.INT32,
            number=2,
        )

    class LlmParser(proto.Message):
        r"""Specifies the LLM parsing for RagFiles.

        Attributes:
            model_name (str):
                The name of a LLM model used for parsing. Format:

                - ``projects/{project_id}/locations/{location}/publishers/{publisher}/models/{model}``
            max_parsing_requests_per_min (int):
                The maximum number of requests the job is
                allowed to make to the LLM model per minute.
                Consult
                https://cloud.google.com/vertex-ai/generative-ai/docs/quotas
                and your document size to set an appropriate
                value here. If unspecified, a default value of
                5000 QPM would be used.
            custom_parsing_prompt (str):
                The prompt to use for parsing. If not
                specified, a default prompt will be used.
        """

        model_name: str = proto.Field(
            proto.STRING,
            number=1,
        )
        max_parsing_requests_per_min: int = proto.Field(
            proto.INT32,
            number=2,
        )
        custom_parsing_prompt: str = proto.Field(
            proto.STRING,
            number=3,
        )

    layout_parser: LayoutParser = proto.Field(
        proto.MESSAGE,
        number=4,
        oneof='parser',
        message=LayoutParser,
    )
    llm_parser: LlmParser = proto.Field(
        proto.MESSAGE,
        number=5,
        oneof='parser',
        message=LlmParser,
    )


class UploadRagFileConfig(proto.Message):
    r"""Config for uploading RagFile.

    Attributes:
        rag_file_transformation_config (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.RagFileTransformationConfig):
            Specifies the transformation config for
            RagFiles.
    """

    rag_file_transformation_config: 'RagFileTransformationConfig' = proto.Field(
        proto.MESSAGE,
        number=3,
        message='RagFileTransformationConfig',
    )


class ImportRagFilesConfig(proto.Message):
    r"""Config for importing RagFiles.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        gcs_source (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.GcsSource):
            Google Cloud Storage location. Supports importing individual
            files as well as entire Google Cloud Storage directories.
            Sample formats:

            - ``gs://bucket_name/my_directory/object_name/my_file.txt``
            - ``gs://bucket_name/my_directory``

            This field is a member of `oneof`_ ``import_source``.
        google_drive_source (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.GoogleDriveSource):
            Google Drive location. Supports importing
            individual files as well as Google Drive
            folders.

            This field is a member of `oneof`_ ``import_source``.
        slack_source (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.SlackSource):
            Slack channels with their corresponding
            access tokens.

            This field is a member of `oneof`_ ``import_source``.
        jira_source (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.JiraSource):
            Jira queries with their corresponding
            authentication.

            This field is a member of `oneof`_ ``import_source``.
        share_point_sources (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.SharePointSources):
            SharePoint sources.

            This field is a member of `oneof`_ ``import_source``.
        partial_failure_gcs_sink (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.GcsDestination):
            The Cloud Storage path to write partial failures to.
            Deprecated. Prefer to use ``import_result_gcs_sink``.

            This field is a member of `oneof`_ ``partial_failure_sink``.
        partial_failure_bigquery_sink (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.BigQueryDestination):
            The BigQuery destination to write partial failures to. It
            should be a bigquery table resource name (e.g.
            "bq://projectId.bqDatasetId.bqTableId"). The dataset must
            exist. If the table does not exist, it will be created with
            the expected schema. If the table exists, the schema will be
            validated and data will be added to this existing table.
            Deprecated. Prefer to use ``import_result_bq_sink``.

            This field is a member of `oneof`_ ``partial_failure_sink``.
        import_result_gcs_sink (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.GcsDestination):
            The Cloud Storage path to write import result
            to.

            This field is a member of `oneof`_ ``import_result_sink``.
        import_result_bigquery_sink (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.BigQueryDestination):
            The BigQuery destination to write import
            result to. It should be a bigquery table
            resource name (e.g.
            "bq://projectId.bqDatasetId.bqTableId"). The
            dataset must exist. If the table does not exist,
            it will be created with the expected schema. If
            the table exists, the schema will be validated
            and data will be added to this existing table.

            This field is a member of `oneof`_ ``import_result_sink``.
        rag_file_transformation_config (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.RagFileTransformationConfig):
            Specifies the transformation config for
            RagFiles.
        rag_file_parsing_config (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.RagFileParsingConfig):
            Optional. Specifies the parsing config for
            RagFiles. RAG will use the default parser if
            this field is not set.
        max_embedding_requests_per_min (int):
            Optional. The max number of queries per
            minute that this job is allowed to make to the
            embedding model specified on the corpus. This
            value is specific to this job and not shared
            across other import jobs. Consult the Quotas
            page on the project to set an appropriate value
            here. If unspecified, a default value of 1,000
            QPM would be used.
        rebuild_ann_index (bool):
            Rebuilds the ANN index to optimize for recall on the
            imported data. Only applicable for RagCorpora running on
            RagManagedDb with ``retrieval_strategy`` set to ``ANN``. The
            rebuild will be performed using the existing ANN config set
            on the RagCorpus. To change the ANN config, please use the
            UpdateRagCorpus API.

            Default is false, i.e., index is not rebuilt.
    """

    gcs_source: io.GcsSource = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof='import_source',
        message=io.GcsSource,
    )
    google_drive_source: io.GoogleDriveSource = proto.Field(
        proto.MESSAGE,
        number=3,
        oneof='import_source',
        message=io.GoogleDriveSource,
    )
    slack_source: io.SlackSource = proto.Field(
        proto.MESSAGE,
        number=6,
        oneof='import_source',
        message=io.SlackSource,
    )
    jira_source: io.JiraSource = proto.Field(
        proto.MESSAGE,
        number=7,
        oneof='import_source',
        message=io.JiraSource,
    )
    share_point_sources: io.SharePointSources = proto.Field(
        proto.MESSAGE,
        number=13,
        oneof='import_source',
        message=io.SharePointSources,
    )
    partial_failure_gcs_sink: io.GcsDestination = proto.Field(
        proto.MESSAGE,
        number=11,
        oneof='partial_failure_sink',
        message=io.GcsDestination,
    )
    partial_failure_bigquery_sink: io.BigQueryDestination = proto.Field(
        proto.MESSAGE,
        number=12,
        oneof='partial_failure_sink',
        message=io.BigQueryDestination,
    )
    import_result_gcs_sink: io.GcsDestination = proto.Field(
        proto.MESSAGE,
        number=14,
        oneof='import_result_sink',
        message=io.GcsDestination,
    )
    import_result_bigquery_sink: io.BigQueryDestination = proto.Field(
        proto.MESSAGE,
        number=15,
        oneof='import_result_sink',
        message=io.BigQueryDestination,
    )
    rag_file_transformation_config: 'RagFileTransformationConfig' = proto.Field(
        proto.MESSAGE,
        number=16,
        message='RagFileTransformationConfig',
    )
    rag_file_parsing_config: 'RagFileParsingConfig' = proto.Field(
        proto.MESSAGE,
        number=8,
        message='RagFileParsingConfig',
    )
    max_embedding_requests_per_min: int = proto.Field(
        proto.INT32,
        number=5,
    )
    rebuild_ann_index: bool = proto.Field(
        proto.BOOL,
        number=19,
    )


class RagManagedDbConfig(proto.Message):
    r"""Configuration message for RagManagedDb used by RagEngine.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        scaled (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.RagManagedDbConfig.Scaled):
            Deprecated: Use ``mode`` instead to set the tier under
            Spanner. Sets the RagManagedDb to the Scaled tier.

            This field is a member of `oneof`_ ``tier``.
        basic (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.RagManagedDbConfig.Basic):
            Deprecated: Use ``mode`` instead to set the tier under
            Spanner. Sets the RagManagedDb to the Basic tier.

            This field is a member of `oneof`_ ``tier``.
        unprovisioned (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.RagManagedDbConfig.Unprovisioned):
            Deprecated: Use ``mode`` instead to set the tier under
            Spanner. Sets the RagManagedDb to the Unprovisioned tier.

            This field is a member of `oneof`_ ``tier``.
        serverless (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.RagManagedDbConfig.Serverless):
            Sets the backend to be the serverless mode
            offered by RAG Engine.

            This field is a member of `oneof`_ ``mode``.
        spanner (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.RagManagedDbConfig.Spanner):
            Sets the RAG Engine backend to be
            RagManagedDb, built on top of Spanner.
            NOTE: This is the default mode (w/ Basic Tier)
            if not explicitly chosen.

            This field is a member of `oneof`_ ``mode``.
    """

    class Scaled(proto.Message):
        r"""Scaled tier offers production grade performance along with
        autoscaling functionality. It is suitable for customers with
        large amounts of data or performance sensitive workloads.

        """

    class Basic(proto.Message):
        r"""Basic tier is a cost-effective and low compute tier suitable for the
        following cases:

        - Experimenting with RagManagedDb.
        - Small data size.
        - Latency insensitive workload.
        - Only using RAG Engine with external vector DBs.

        NOTE: This is the default tier under Spanner mode if not explicitly
        chosen.

        """

    class Unprovisioned(proto.Message):
        r"""Disables the RAG Engine service and deletes all your data
        held within this service. This will halt the billing of the
        service.

        NOTE: Once deleted the data cannot be recovered. To start using
        RAG Engine again, you will need to update the tier by calling
        the UpdateRagEngineConfig API.

        """

    class Spanner(proto.Message):
        r"""Message to configure the Spanner database used by
        RagManagedDb.

        This message has `oneof`_ fields (mutually exclusive fields).
        For each oneof, at most one member field can be set at the same time.
        Setting any member of the oneof automatically clears all other
        members.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            scaled (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.RagManagedDbConfig.Scaled):
                Sets the RagManagedDb to the Scaled tier.

                This field is a member of `oneof`_ ``tier``.
            basic (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.RagManagedDbConfig.Basic):
                Sets the RagManagedDb to the Basic tier. This
                is the default tier for Spanner mode if not
                explicitly chosen.

                This field is a member of `oneof`_ ``tier``.
            unprovisioned (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.RagManagedDbConfig.Unprovisioned):
                Sets the RagManagedDb to the Unprovisioned
                tier.

                This field is a member of `oneof`_ ``tier``.
        """

        scaled: 'RagManagedDbConfig.Scaled' = proto.Field(
            proto.MESSAGE,
            number=1,
            oneof='tier',
            message='RagManagedDbConfig.Scaled',
        )
        basic: 'RagManagedDbConfig.Basic' = proto.Field(
            proto.MESSAGE,
            number=2,
            oneof='tier',
            message='RagManagedDbConfig.Basic',
        )
        unprovisioned: 'RagManagedDbConfig.Unprovisioned' = proto.Field(
            proto.MESSAGE,
            number=3,
            oneof='tier',
            message='RagManagedDbConfig.Unprovisioned',
        )

    class Serverless(proto.Message):
        r"""Message to configure the serverless mode offered by RAG
        Engine.

        """

    scaled: Scaled = proto.Field(
        proto.MESSAGE,
        number=4,
        oneof='tier',
        message=Scaled,
    )
    basic: Basic = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof='tier',
        message=Basic,
    )
    unprovisioned: Unprovisioned = proto.Field(
        proto.MESSAGE,
        number=3,
        oneof='tier',
        message=Unprovisioned,
    )
    serverless: Serverless = proto.Field(
        proto.MESSAGE,
        number=5,
        oneof='mode',
        message=Serverless,
    )
    spanner: Spanner = proto.Field(
        proto.MESSAGE,
        number=6,
        oneof='mode',
        message=Spanner,
    )


class RagEngineConfig(proto.Message):
    r"""Config for RagEngine.

    Attributes:
        name (str):
            Identifier. The name of the RagEngineConfig. Format:
            ``projects/{project}/locations/{location}/ragEngineConfig``
        rag_managed_db_config (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.RagManagedDbConfig):
            The config of the RagManagedDb used by
            RagEngine.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    rag_managed_db_config: 'RagManagedDbConfig' = proto.Field(
        proto.MESSAGE,
        number=2,
        message='RagManagedDbConfig',
    )


__all__ = tuple(sorted(__protobuf__.manifest))
